@@ -1,6 +1,7 @@
 // State management for Pynecone web apps.
 import axios from "axios";
 import io from "socket.io-client";
+import JSON5 from "json5";
 
 // Global variable to hold the token.
 let token;
@@ -175,10 +176,10 @@ export const connect = async (
 
   // On each received message, apply the delta and set the result.
   socket.current.on("event", function (update) {
-    update = JSON.parse(update);
+    update = JSON5.parse(update);
     applyDelta(state, update.delta);
     setResult({
-      processing: false,
+      processing: true,
       state: state,
       events: update.events,
     });
@@ -192,6 +193,7 @@ export const connect = async (
  * @param setResult The function to set the result.
  * @param files The files to upload.
  * @param handler The handler to use.
+ * @param multiUpload Whether handler args on backend is multiupload
  * @param endpoint The endpoint to upload to.
  */
 export const uploadFiles = async (
@@ -210,15 +212,15 @@ export const uploadFiles = async (
   // Set processing to true to block other events from being processed.
   setResult({ ...result, processing: true });
 
-  // Currently only supports uploading one file.
-  const file = files[0];
   const headers = {
-    "Content-Type": file.type,
+    "Content-Type": files[0].type,
   };
   const formdata = new FormData();
 
   // Add the token and handler to the file name.
-  formdata.append("file", file, getToken() + ":" + handler + ":" + file.name);
+  for (let i = 0; i < files.length; i++) {
+    formdata.append("files", files[i], getToken() + ":" + handler + ":" + files[i].name);
+  }
 
   // Send the file to the server.
   await axios.post(endpoint, formdata, headers).then((response) => {
