@@ -3,12 +3,13 @@
 Anything imported here will be available in the default Pynecone import as `pc.*`.
 """
 
+from . import el
 from .app import App, UploadFile
 from .base import Base
 from .components import *
-from .components.component import custom_component as component
+from .components.component import custom_component as memo
 from .components.graphing.victory import data
-from .config import Config
+from .config import Config, DBConfig
 from .constants import Env, Transports
 from .event import (
     EVENT_ARG,
