@@ -32,6 +32,9 @@ class Plotly(PlotlyLib):
     # The height of the graph.
     height: Var[str]
 
+    # If true, the graph will resize when the window is resized.
+    use_resize_handler: Var[bool]
+
     def _get_imports(self):
         return {}
 
