@@ -17,6 +17,11 @@ VERSION = pkg_resources.get_distribution(PACKAGE_NAME).version
 # Minimum version of Node.js required to run Pynecone.
 MIN_NODE_VERSION = "12.22.0"
 
+# Valid bun versions.
+MIN_BUN_VERSION = "0.5.5"
+MAX_BUN_VERSION = "0.5.8"
+INVALID_BUN_VERSIONS = ["0.5.6", "0.5.7"]
+
 # Files and directories used to init a new project.
 # The root directory of the pynecone library.
 ROOT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
@@ -80,6 +85,9 @@ TIMEOUT = 120
 # The command to run the backend in production mode.
 RUN_BACKEND_PROD = f"gunicorn --worker-class uvicorn.workers.UvicornH11Worker --preload --timeout {TIMEOUT} --log-level critical".split()
 RUN_BACKEND_PROD_WINDOWS = f"uvicorn --timeout-keep-alive {TIMEOUT}".split()
+# Socket.IO web server
+PING_INTERVAL = 25
+PING_TIMEOUT = 5
 
 # Compiler variables.
 # The extension for compiled Javascript files.
@@ -185,10 +193,10 @@ class Endpoint(Enum):
             The full URL for the endpoint.
         """
         # Import here to avoid circular imports.
-        from pynecone import utils
+        from pynecone.config import get_config
 
         # Get the API URL from the config.
-        config = utils.get_config()
+        config = get_config()
         url = "".join([config.api_url, str(self)])
 
         # The event endpoint is a websocket.
@@ -238,10 +246,10 @@ class Transports(Enum):
             The transports config for the backend.
         """
         # Import here to avoid circular imports.
-        from pynecone import utils
+        from pynecone.config import get_config
 
-        # Get the transports from the config.
-        config = utils.get_config()
+        # Get the API URL from the config.
+        config = get_config()
         return str(config.backend_transports)
 
 
