@@ -21,9 +21,10 @@ from typing import (
 import cloudpickle
 from redis import Redis
 
-from pynecone import constants, utils
+from pynecone import constants
 from pynecone.base import Base
-from pynecone.event import Event, EventHandler, window_alert
+from pynecone.event import Event, EventHandler, fix_events, window_alert
+from pynecone.utils import format, prerequisites, types
 from pynecone.var import BaseVar, ComputedVar, PCDict, PCList, Var
 
 Delta = Dict[str, Any]
@@ -50,6 +51,9 @@ class State(Base, ABC):
     # Backend vars inherited
     inherited_backend_vars: ClassVar[Dict[str, Any]] = {}
 
+    # The event handlers.
+    event_handlers: ClassVar[Dict[str, EventHandler]] = {}
+
     # The parent state.
     parent_state: Optional[State] = None
 
@@ -93,7 +97,7 @@ class State(Base, ABC):
                 value, self._reassign_field, field.name
             )
 
-            if utils._issubclass(field.type_, Union[List, Dict]):
+            if types._issubclass(field.type_, Union[List, Dict]):
                 setattr(self, field.name, value_in_pc_data)
 
         self.clean()
@@ -138,7 +142,7 @@ class State(Base, ABC):
         cls.new_backend_vars = {
             name: value
             for name, value in cls.__dict__.items()
-            if utils.is_backend_variable(name)
+            if types.is_backend_variable(name)
             and name not in cls.inherited_backend_vars
         }
 
@@ -180,8 +184,18 @@ class State(Base, ABC):
         }
         for name, fn in events.items():
             event_handler = EventHandler(fn=fn)
+            cls.event_handlers[name] = event_handler
             setattr(cls, name, event_handler)
 
+    @classmethod
+    def convert_handlers_to_fns(cls):
+        """Convert the event handlers to functions.
+
+        This is done so the state functions can be called as normal functions during runtime.
+        """
+        for name, event_handler in cls.event_handlers.items():
+            setattr(cls, name, event_handler.fn)
+
     @classmethod
     @functools.lru_cache()
     def get_parent_state(cls) -> Optional[Type[State]]:
@@ -193,7 +207,7 @@ class State(Base, ABC):
         parent_states = [
             base
             for base in cls.__bases__
-            if utils._issubclass(base, State) and base is not State
+            if types._issubclass(base, State) and base is not State
         ]
         assert len(parent_states) < 2, "Only one parent state is allowed."
         return parent_states[0] if len(parent_states) == 1 else None  # type: ignore
@@ -216,7 +230,7 @@ class State(Base, ABC):
         Returns:
             The name of the state.
         """
-        return utils.to_snake_case(cls.__name__)
+        return format.to_snake_case(cls.__name__)
 
     @classmethod
     @functools.lru_cache()
@@ -286,7 +300,7 @@ class State(Base, ABC):
         Raises:
             TypeError: if the variable has an incorrect type
         """
-        if not utils.is_valid_var_type(prop.type_):
+        if not types.is_valid_var_type(prop.type_):
             raise TypeError(
                 "State vars must be primitive Python types, "
                 "Plotly figures, Pandas dataframes, "
@@ -482,7 +496,7 @@ class State(Base, ABC):
             setattr(self.parent_state, name, value)
             return
 
-        if utils.is_backend_variable(name):
+        if types.is_backend_variable(name):
             self.backend_vars.__setitem__(name, value)
             self.mark_dirty()
             return
@@ -532,37 +546,65 @@ class State(Base, ABC):
         return self.substates[path[0]].get_substate(path[1:])
 
     async def process(self, event: Event) -> StateUpdate:
-        """Process an event.
+        """Obtain event info and process event.
 
         Args:
             event: The event to process.
 
         Returns:
             The state update after processing the event.
+
+        Raises:
+            ValueError: If the state value is None.
         """
         # Get the event handler.
         path = event.name.split(".")
         path, name = path[:-1], path[-1]
         substate = self.get_substate(path)
-        handler = getattr(substate, name)
+        handler = substate.event_handlers[name]  # type: ignore
 
-        # Process the event.
-        fn = functools.partial(handler.fn, substate)
+        if not substate:
+            raise ValueError(
+                "The value of state cannot be None when processing an event."
+            )
+
+        return await self.process_event(
+            handler=handler,
+            state=substate,
+            payload=event.payload,
+            token=event.token,
+        )
+
+    async def process_event(
+        self, handler: EventHandler, state: State, payload: Dict, token: str
+    ) -> StateUpdate:
+        """Process event.
+
+        Args:
+            handler: Eventhandler to process.
+            state: State to process the handler.
+            payload: The event payload.
+            token: Client token.
+
+        Returns:
+            The state update after processing the event.
+        """
+        fn = functools.partial(handler.fn, state)
         try:
             if asyncio.iscoroutinefunction(fn.func):
-                events = await fn(**event.payload)
+                events = await fn(**payload)
             else:
-                events = fn(**event.payload)
+                events = fn(**payload)
         except Exception:
             error = traceback.format_exc()
             print(error)
-            events = utils.fix_events(
-                [window_alert("An error occurred. See logs for details.")], event.token
+            events = fix_events(
+                [window_alert("An error occurred. See logs for details.")], token
             )
             return StateUpdate(events=events)
 
         # Fix the returned events.
-        events = utils.fix_events(events, event.token)
+        events = fix_events(events, token)
 
         # Get the delta after processing the event.
         delta = self.get_delta()
@@ -595,7 +637,7 @@ class State(Base, ABC):
             delta.update(substates[substate].get_delta())
 
         # Format the delta.
-        delta = utils.format_state(delta)
+        delta = format.format_state(delta)
 
         # Return the delta.
         return delta
@@ -685,7 +727,7 @@ class StateManager(Base):
             state: The state class to use.
         """
         self.state = state
-        self.redis = utils.get_redis()
+        self.redis = prerequisites.get_redis()
 
     def get_state(self, token: str) -> State:
         """Get the state for a token.
